package com.bytehamster.changelog;

import java.util.Map;
import android.app.Activity;
import android.app.Dialog;
import android.content.ActivityNotFoundException;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.support.v7.app.AlertDialog;

import android.text.Html;
import android.text.SpannableString;
import android.text.method.LinkMovementMethod;
import android.util.TypedValue;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

class Dialogs {
    
    public static void usingCacheAlert(final Activity activity, final String gerritUrl,
                                       final DialogInterface.OnClickListener retryPressedListener){
        activity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                AlertDialog.Builder b = new AlertDialog.Builder(activity);
                b.setTitle(R.string.network_error);
                b.setMessage(R.string.using_cache);
                b.setCancelable(true);

                b.setNegativeButton(R.string.open_website, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        Intent browserIntent = new Intent(Intent.ACTION_VIEW, Uri.parse(gerritUrl));
                        try {
                            activity.startActivity(browserIntent);
                        } catch (ActivityNotFoundException e) {
                            Toast.makeText(activity, "No browser installed", Toast.LENGTH_LONG).show();
                        }
                    }
                });
                b.setPositiveButton(R.string.ok, null);
                b.setNeutralButton(R.string.retry, retryPressedListener);
                b.show();
            }
        });
    }
	
	public static void changeDetails(final Activity activity, final Map<String, Object> change, final String gerrit_url){
		String msg = activity.getString(R.string.change_content);
		msg = msg.replace("%title", (String)change.get("title"));
        msg = msg.replace("%owner", (String)change.get("owner"));
        msg = msg.replace("%author", (String)change.get("author"));
    	msg = msg.replace("%date", (String)change.get("dateFull"));
        msg = msg.replace("%project", (String)change.get("project"));
        msg = msg.replace("%branch", (String)change.get("branch"));
    	
    	String message = (String)change.get("message");
    	message = StringTools.filterCommitMessage(message, activity);
        message = message.replaceAll("(\\A|\\s)((http|https|ftp|mailto):\\S+)(?=\\s|\\z)","$1<a href=\"$2\">$2</a>");
    	msg = msg.replace("%message", message.replace("\n","<br />"));


        final TextView messageView = new TextView(activity);
        final SpannableString s = new SpannableString(Html.fromHtml(msg));
        messageView.setText(s);

        final float scale = activity.getResources().getDisplayMetrics().density;
        int padding_in_px = (int) (24 * scale + 0.5f);

        messageView.setPadding(padding_in_px, padding_in_px, padding_in_px, padding_in_px);
        messageView.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 16);
        messageView.setTextColor(0xff000000);
        messageView.setMovementMethod(LinkMovementMethod.getInstance());


        AlertDialog.Builder d = new AlertDialog.Builder(activity);
        d.setCancelable(true);
        d.setTitle(R.string.change);
        d.setView(messageView);
        d.setNegativeButton("Gerrit", null);
        d.setPositiveButton(R.string.ok, null);
        Dialog dlg = d.create();
        dlg.setCanceledOnTouchOutside(true);

        dlg.setOnShowListener(new DialogInterface.OnShowListener() {
            @Override
            public void onShow(DialogInterface dialog) {
                Button button = ((AlertDialog) dialog).getButton(AlertDialog.BUTTON_NEGATIVE);
                button.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        Uri uri = Uri.parse(gerrit_url + "#/c/" + change.get("number"));
                        Intent intent = new Intent(Intent.ACTION_VIEW, uri);
                        try {
                            activity.startActivity(intent);
                        } catch (ActivityNotFoundException e) {
                            Toast.makeText(activity, "No browser installed", Toast.LENGTH_LONG).show();
                        }
                    }
                });
            }
        });

        dlg.show();
	}
}