package com.bytehamster.changelog;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.text.Editable;
import android.text.Html;
import android.text.TextWatcher;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.AbsListView;
import android.widget.AdapterView;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.TextView;
import android.widget.Toast;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.StringReader;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

public class Main extends AppCompatActivity {
    static final String                          DEFAULT_GERRIT_URL   = "https://gerrit.omnirom.org/";
    static final String                          DEFAULT_BRANCH       = "";
    static final int                             MAX_CHANGES          = 200;
    static final int                             MAX_CHANGES_FETCH    = 800;  // Max changes to be fetched
    static final int                             MAX_CHANGES_DB       = 1500; // Max changes to be loaded from DB
    static final SimpleDateFormat                mDateFormat          = new SimpleDateFormat("yyyy-MM-dd HH:mm", Locale.US);
    static final SimpleDateFormat                mDateDayFormat       = new SimpleDateFormat("EEE, dd MMM yyyy", Locale.US);

    private final ArrayList<Map<String, Object>> mChangesList         = new ArrayList<>();
    private final List<HashMap<String, Object>>  mWatchedList         = new ArrayList<>();
    private ArrayList<Map<String, Object>>       mDevicesList         = new ArrayList<>();

    private ListView                             mListView            = null;
    private Activity                             mActivity            = null;
    private SwipeRefreshLayout                   swipeContainer       = null;
    private SharedPreferences                    mSharedPreferences   = null;
    private String                               mDeviceFilterKeyword = "";
    private String                               mLastDate            = "";
    private boolean                              mIsLoading           = false;
    private boolean                              mJustStarted         = true;
    private Document                             mWatchedDoc          = null;
    private ChangeAdapter                        mChangeAdapter       = null;
    private int                                  mChangesCount        = 0;
    private String                               GERRIT_URL           = "https://gerrit.omnirom.org/";
    private boolean                              isScrolledToTop      = true;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        mActivity = this;
        getSupportActionBar().setTitle(R.string.changelog);
        mSharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        GERRIT_URL = mSharedPreferences.getString("server_url", DEFAULT_GERRIT_URL);
        mChangeAdapter = new ChangeAdapter(mActivity, mChangesList, GERRIT_URL);
        mListView = findViewById(android.R.id.list);

        mListView.setAdapter(mChangeAdapter);
        mListView.setOnItemClickListener(MainListClickListener);
        mListView.setOnItemLongClickListener(MainListLongClickListener);
        swipeContainer = findViewById(R.id.swipe_container);
        swipeContainer.setRefreshing(true);
        swipeContainer.setOnRefreshListener(() -> load(false));
        mListView.setOnScrollListener(new AbsListView.OnScrollListener() {
            @Override
            public void onScrollStateChanged(AbsListView view, int scrollState) { }

            @Override
            public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount, int totalItemCount) {
                int topRowVerticalPosition =
                        (mListView == null || mListView.getChildCount() == 0) ?
                                0 : mListView.getChildAt(0).getTop();
                isScrolledToTop = firstVisibleItem == 0 && topRowVerticalPosition >= 0;
                if (!mIsLoading) {
                    swipeContainer.setEnabled(isScrolledToTop);
                }
            }
        });

        if (mSharedPreferences.getString("branch", DEFAULT_BRANCH).equals("All")) {
            mSharedPreferences.edit().putString("branch", "").apply();
        }

		load(true);
		checkAlerts();
	}

    @Override
    protected void onStart() {
        super.onStart();
        GERRIT_URL = mSharedPreferences.getString("server_url", DEFAULT_GERRIT_URL);

        if (mJustStarted) {
            mJustStarted = false;
        } else {
            load(false);
        }
    }

    private void checkAlerts() {
        if (!mSharedPreferences.getBoolean("warning_displayed", false)) {
            AlertDialog.Builder d = new AlertDialog.Builder(mActivity);
            d.setCancelable(false);
            d.setTitle(R.string.first_warning);
            d.setMessage(Html.fromHtml(getResources().getString(R.string.first_warning_message)));
            d.setPositiveButton(R.string.ok, (dialog, which) ->
                    mSharedPreferences.edit().putBoolean("warning_displayed", true).apply());
            d.show();
        }

        if ((!Build.DISPLAY.contains("omni") || !Build.DISPLAY.contains("lineage"))
                && !mSharedPreferences.getBoolean("openApp", false)) {
            AlertDialog.Builder d = new AlertDialog.Builder(mActivity);
            d.setCancelable(false);
            d.setTitle(R.string.not_supported);
            d.setMessage(Html.fromHtml(getResources().getString(R.string.not_supported_content)));
            d.setNegativeButton(R.string.close, (dialog, which) -> finish());
            d.setPositiveButton(R.string.open, (dialog, which) ->
                    mSharedPreferences.edit().putBoolean("openApp", true).apply());
            d.show();
        }
    }

    private void load(final boolean loadCachedFirst) {
        if (mIsLoading) {
            return;
        }
        mIsLoading = true;
        swipeContainer.setRefreshing(true);

        new Thread(() -> {
            if (!mChangesList.isEmpty()) mChangesList.clear();

            ChangeLoader loader = new ChangeLoader(mActivity, mSharedPreferences, GERRIT_URL);
            ChangeFilter filter = new ChangeFilter(mSharedPreferences);

            List<Change> changes;
            if (loadCachedFirst) {
                changes = loader.loadCached();
            } else {
                try {
                    changes = loader.loadAll();
                } catch (ChangeLoader.LoadException e) {
                    Dialogs.usingCacheAlert(mActivity, GERRIT_URL, (dialogInterface, i) -> load(true));
                    changes = loader.getCached();
                }
            }

            mChangesCount = 0;
            mLastDate = "-";
            for (Change currentChange : changes) {
                if (filter.isHidden(currentChange)) {
                    continue;
                }

                if (!mLastDate.equals(currentChange.dateDay)) {
                    Map<String, Object> new_item = new HashMap<>();
                    new_item.put("title", currentChange.dateDay);
                    new_item.put("type", Change.TYPE_HEADER);
                    mChangesList.add(new_item);
                    mLastDate = currentChange.dateDay;
                }
                mChangesList.add(currentChange.getHashMap(mActivity));
                mChangesCount++;

                if (mChangesCount >= MAX_CHANGES) {
                    break;
                }
            }

            mActivity.runOnUiThread(() -> {
                showEmptyIfNecessary();
                ((TextView) findViewById(android.R.id.empty)).setText(R.string.no_changes);
                mChangeAdapter.update(mChangesList);
                if (mChangesCount >= MAX_CHANGES) {
                    getSupportActionBar().setTitle(getResources().getString(R.string.changelog) + " (" + MAX_CHANGES + "+)");
                } else {
                    getSupportActionBar().setTitle(getResources().getString(R.string.changelog) + " (" + mChangesCount + ")");
                }
                mIsLoading = false;

                if (loadCachedFirst) {
                    load(false);
                } else {
                    swipeContainer.setRefreshing(false);
                    swipeContainer.setEnabled(isScrolledToTop);
                }
            });
        }).start();
    }

    private void showEmptyIfNecessary(){
        findViewById(android.R.id.empty).setVisibility(View.GONE);
        if (mChangesList.isEmpty()) {
            findViewById(android.R.id.empty).setVisibility(View.VISIBLE);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
            case R.id.action_info:

                String msg = getResources().getString(R.string.info_content);
                msg = msg.replace("%buildtime", mDateFormat.format(Build.TIME));
                msg = msg.replace("%lastrefresh", mDateFormat.format(mSharedPreferences.getLong("lastRefresh", 0)));

                AlertDialog.Builder d = new AlertDialog.Builder(mActivity);
                d.setCancelable(true);
                d.setTitle(R.string.info);
                d.setMessage(Html.fromHtml(msg));
                d.setPositiveButton(R.string.ok, null);
                d.show().setCanceledOnTouchOutside(true);
                return true;
            case R.id.action_refresh:
                load(false);
                return true;
            case R.id.action_filter:
                filter();
                return true;
            case R.id.action_settings:
                Intent i = new Intent(this, Preferences.class);
                startActivity(i);
                return true;
            case R.id.action_feedback:
                AlertDialog.Builder d2 = new AlertDialog.Builder(mActivity);
                d2.setCancelable(true);
                d2.setTitle(R.string.feedback);
                d2.setMessage(R.string.feedback_warning);
                d2.setNegativeButton(R.string.cancel, null);
                d2.setPositiveButton(R.string.ok, (dialog, which) ->
                        External.feedbackMail(mActivity, getString(R.string.feedback_title), ""));
                d2.show().setCanceledOnTouchOutside(true);
                return true;
            default:
                return super.onOptionsItemSelected(item);
	    }
    }

    private void filter() {

        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db;
            db = dbf.newDocumentBuilder();
            InputSource is = new InputSource();
            is.setCharacterStream(new StringReader(mSharedPreferences.getString("watched_devices", "<?xml version=\"1.0\" encoding=\"UTF-8\"?><devicesList></devicesList>")));
            mWatchedDoc = db.parse(is);
            mWatchedDoc.getDocumentElement().normalize();
        }
        catch (Exception e) {
            e.printStackTrace();
        }

        final AlertDialog.Builder b = new AlertDialog.Builder(this);
        b.setCancelable(true);
        b.setTitle(R.string.filter);
        final View root = View.inflate(this, R.layout.dialog_filter, null);
        b.setView(root);


        b.setOnDismissListener(dialog -> load(true));
        b.setPositiveButton(R.string.ok, null);

        final Dialog d = b.create();

        d.setCanceledOnTouchOutside(true);

        ((CheckBox) root.findViewById(R.id.translations)).setChecked(mSharedPreferences.getBoolean("translations", true));
        ((CheckBox) root.findViewById(R.id.show_twrp)).setChecked(mSharedPreferences.getBoolean("show_twrp", true));


        final EditText branch = root.findViewById(R.id.branch);
        branch.setText(mSharedPreferences.getString("branch", DEFAULT_BRANCH));
        branch.addTextChangedListener(new TextWatcher() {
            public void afterTextChanged(Editable s) {
                mSharedPreferences.edit().putString("branch", s.toString()).apply();
            }
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {}
            public void onTextChanged(CharSequence s, int start, int before, int count) {}
        });


        if (mSharedPreferences.getBoolean("display_all", true)) {
            root.findViewById(R.id.devices_listview).setVisibility(View.GONE);
            root.findViewById(R.id.add_device).setVisibility(View.GONE);
            ((CheckBox) root.findViewById(R.id.all_devices)).setChecked(true);
        } else {
            ((CheckBox) root.findViewById(R.id.all_devices)).setChecked(false);
            loadDeviceList(root.findViewById(R.id.devices_listview));
        }

        ((ListView) root.findViewById(R.id.devices_listview)).setOnItemClickListener((arg0, view, pos, id) -> {
            mWatchedDoc.getDocumentElement().removeChild((Element) mWatchedList.get(pos).get("device_element"));
            mSharedPreferences.edit().putString("watched_devices", StringTools.XmlToString(mActivity, mWatchedDoc)).apply();
            loadDeviceList(root.findViewById(R.id.devices_listview));
        });
        ((CheckBox) root.findViewById(R.id.translations)).setOnCheckedChangeListener((buttonView, isChecked) ->
                mSharedPreferences.edit().putBoolean("translations", isChecked).apply());
        ((CheckBox) root.findViewById(R.id.show_twrp)).setOnCheckedChangeListener((buttonView, isChecked) ->
                mSharedPreferences.edit().putBoolean("show_twrp", isChecked).apply());
        ((CheckBox) root.findViewById(R.id.all_devices)).setOnCheckedChangeListener((buttonView, isChecked) -> {
            if (isChecked) {
                mSharedPreferences.edit().putBoolean("display_all", true).apply();
                root.findViewById(R.id.devices_listview).setVisibility(View.GONE);
                root.findViewById(R.id.add_device).setVisibility(View.GONE);
            } else {
                mSharedPreferences.edit().putBoolean("display_all", false).apply();
                root.findViewById(R.id.devices_listview).setVisibility(View.VISIBLE);
                root.findViewById(R.id.add_device).setVisibility(View.VISIBLE);
                if (mSharedPreferences.getString("watched_devices", "").equals("")) {
                    mSharedPreferences.edit().putString("watched_devices", "<?xml version=\"1.0\" encoding=\"UTF-8\"?><devicesList></devicesList>").apply();
                }
                loadDeviceList(root.findViewById(R.id.devices_listview));
            }
        });
        root.findViewById(R.id.add_device).setOnClickListener(v -> addDevice(root.findViewById(R.id.devices_listview)));
        d.show();
    }

    private void addDevice(final ListView v) {

        mDeviceFilterKeyword = "";

        final AlertDialog.Builder b = new AlertDialog.Builder(this);
        b.setCancelable(true);
        b.setTitle(R.string.add_device);
        b.setPositiveButton(R.string.cancel, null);
        final View root = View.inflate(this, R.layout.dialog_add_device, null);
        b.setView(root);

        final Dialog d = b.create();
        d.setCanceledOnTouchOutside(true);


        ((EditText)root.findViewById(R.id.search_value)).setOnEditorActionListener((v1, actionId, event) -> {
            if (actionId == EditorInfo.IME_ACTION_SEARCH) {
                InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.hideSoftInputFromWindow(v1.getWindowToken(), 0);
                mDeviceFilterKeyword = ((EditText) root.findViewById(R.id.search_value)).getText().toString().trim();
                loadAllDeviceList(root.findViewById(R.id.devices_listview));
            }
            return false;
        });

        root.findViewById(R.id.report_missing_device).setOnClickListener(v12 -> External.requestNewDevice(Main.this));
        root.findViewById(R.id.search_button).setOnClickListener(v13 -> {
            mDeviceFilterKeyword = ((EditText) root.findViewById(R.id.search_value)).getText().toString().trim();
            loadAllDeviceList(root.findViewById(R.id.devices_listview));
        });
        ((ListView) root.findViewById(R.id.devices_listview)).setOnItemClickListener((arg0, view, pos, id) -> {
            Node new_node = mWatchedDoc.importNode((Element) mDevicesList.get(pos).get("device_element"), true);
            mWatchedDoc.getDocumentElement().appendChild(new_node);
            mSharedPreferences.edit().putString("watched_devices", StringTools.XmlToString(mActivity, mWatchedDoc)).apply();

            d.dismiss();
            loadDeviceList(v);
        });

        d.show();

        loadAllDeviceList(root.findViewById(R.id.devices_listview));
    }

    private void loadAllDeviceList(ListView listView) {
        mDevicesList = Devices.loadDefinitions(this, mDeviceFilterKeyword);

        if (mDevicesList == null) {
            Toast.makeText(this, "Error while loading devices", Toast.LENGTH_LONG).show();
            return;
        }

        SimpleAdapter sAdapter = new SimpleAdapter(mActivity, mDevicesList, R.layout.list_entry_device, new String[] { "name", "code", "code" }, new int[] { R.id.name, R.id.code, R.id.aside });
        sAdapter.setViewBinder(all_devices_view_binder);
        listView.setAdapter(sAdapter);
    }

    private final SimpleAdapter.ViewBinder all_devices_view_binder = (view, data, textRepresentation) -> {
        if (view.getId() == R.id.aside) {
            if (Build.DEVICE.toLowerCase(Locale.getDefault()).equals(textRepresentation) ||
                    Build.MODEL.toLowerCase(Locale.getDefault()).replace("gt-", "").equals(textRepresentation)) {
                ((TextView) view).setText(R.string.this_device);
            } else ((TextView) view).setText("");
            return true;
        }
        return false;
    };

    private void loadDeviceList(final ListView listView) {
        if (mWatchedDoc == null) {
            // Not loaded. Try again later.
            new Handler().postDelayed( new Runnable() {
                public void run() {
                    loadDeviceList(listView);
                }
            }, 500);
            return;
        }
        if (!mWatchedList.isEmpty()) mWatchedList.clear();

        HashMap<String, Object> AddItemMap;

        NodeList devicesList = mWatchedDoc.getDocumentElement().getChildNodes();
        for (int i = 0; i < devicesList.getLength(); i++) {
            if (devicesList.item(i).getNodeType() != Node.ELEMENT_NODE) continue;

            Element device = (Element) devicesList.item(i);
            AddItemMap = new HashMap<>();
            NodeList properties = device.getChildNodes();
            AddItemMap.put("device_element", device);

            for (int k = 0; k < properties.getLength(); k++) {
                if (properties.item(k).getNodeType() != Node.ELEMENT_NODE) continue;
                Element property = (Element) properties.item(k);

                if (property.getNodeName().equals("name")) AddItemMap.put("name", property.getTextContent());

            }
            mWatchedList.add(AddItemMap);
        }

        Collections.sort(mWatchedList, new Devices.Comparator());

        SimpleAdapter sAdapter = new SimpleAdapter(mActivity, mWatchedList, android.R.layout.simple_list_item_1,
                new String[] { "name" }, new int[] { android.R.id.text1 });

        listView.setAdapter(sAdapter);
    }

    private final AdapterView.OnItemLongClickListener MainListLongClickListener = (parent, view, position, id) -> {
        if ((Integer) mChangesList.get(position).get("type") == Change.TYPE_ITEM) {
            AlertDialog.Builder d = new AlertDialog.Builder(mActivity);
            d.setCancelable(true);
            d.setTitle(R.string.change);
            d.setMessage((String) mChangesList.get(position).get("title"));
            d.setNegativeButton(R.string.cancel, null);
            d.setPositiveButton("Gerrit", (dialog, which) -> {
                Uri uri = Uri.parse(GERRIT_URL + "#/c/" + mChangesList.get(position).get("number"));
                Intent intent = new Intent(Intent.ACTION_VIEW, uri);
                startActivity(intent);
            });
            d.show().setCanceledOnTouchOutside(true);
        }

        return true;
    };

    private final AdapterView.OnItemClickListener MainListClickListener = (parent, view, position, id) -> {
        if (((Integer) mChangeAdapter.getItem(position).get("type")) == Change.TYPE_ITEM) {

            mChangeAdapter.getItem(position).put("is_new", false);
            view.findViewById(R.id.is_new).setVisibility(View.GONE);

            if (mSharedPreferences.getString("list_action", "popup").equals("popup")) {
                Dialogs.changeDetails(mActivity, mChangeAdapter.getItem(position), GERRIT_URL);
            } else {
                final TextView info = view.findViewById(R.id.info);
                final View buttons = view.findViewById(R.id.buttons);
                final TextView title = view.findViewById(R.id.title);

                if (info.getVisibility() == View.GONE) {
                    title.setMaxLines(20);
                    info.setVisibility(View.VISIBLE);
                    buttons.setVisibility(View.VISIBLE);
                    mChangeAdapter.getItem(position).put("visibility", View.VISIBLE);
                } else {
                    title.setMaxLines(1);
                    info.setVisibility(View.GONE);
                    buttons.setVisibility(View.GONE);
                    mChangeAdapter.getItem(position).put("visibility", View.GONE);
                }
            }
        }
    };
}
